package manas.event;

import org.jgraph.graph.*;
import org.jgraph.JGraph;

import javax.swing.*;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.event.MouseEvent;
import java.awt.*;
import java.util.Map;
import java.util.Hashtable;

import manas.GGates.GSwitch;
import manas.GGates.GClock;

/**
 * Created by IntelliJ IDEA.
 * User: SahinBey
 * Date: 25.Nis.2006
 * Time: 08:05:23
 * To change this template use File | Settings | File Templates.
 */
// MarqueeHandler that Connects Vertices and Displays PopupMenus
public class MyMarqueeHandler extends BasicMarqueeHandler {
    JGraph graph;

    public MyMarqueeHandler(JGraph graph) {
        this.graph = graph;
    }

    // Holds the Start and the Current Point
    protected Point2D start, current;

    // Holds the First and the Current Port
    protected PortView port, firstPort;

    // Override to Gain Control (for PopupMenu and ConnectMode)
    public boolean isForceMarqueeEvent(MouseEvent e) {
        if (e.isShiftDown())
            return false;
        // If Right Mouse Button we want to Display the PopupMenu
        if (SwingUtilities.isRightMouseButton(e))
            // Return Immediately
            return true;
        // Find and Remember Port
        port = getSourcePortAt(e.getPoint());
        // If Port Found and in ConnectMode (=Ports Visible)
        if (port != null && graph.isPortsVisible())
            return true;
        // Else Call Superclass
        return super.isForceMarqueeEvent(e);
    }

    // Display PopupMenu or Remember Start Location and First Port
    public void mousePressed(final MouseEvent e) {
        // If Right Mouse Button
        if (SwingUtilities.isRightMouseButton(e)) {
            // Find Cell in Model Coordinates
            Object cell = graph.getFirstCellForLocation(e.getX(), e.getY());
            int ind = graph.getModel().getIndexOfRoot(cell);
//            System.out.println("secilen cell indexi = " + ind);
            if(cell instanceof GSwitch) {
                ((GSwitch)cell).changeState();
//                graph.getGraphLayoutCache().refresh(graph.getGraphLayoutCache().getCellViews(), true);
//                graph.getGraphLayoutCache().update(graph.getGraphLayoutCache().getCellViews());
               /* Graphics g = graph.getGraphics();
                PortView pv = graph.getPortViewAt(e.getX(), e.getY());
                graph.getUI().paintCell(g, pv, pv.getBounds(), true);
                */
                        // If remembered Start Point is Valid
            } else if(cell instanceof GClock) {
                if(((GClock)cell).isActive()) ((GClock)cell).setActive(false);
                else ((GClock)cell).setActive(true);
            }

        if (start != null) {
            // Fetch Graphics from Graph
            Graphics g = graph.getGraphics();
            // Reset Remembered Port
            PortView newPort = getTargetPortAt(e.getPoint());
            // Do not flicker (repaint only on real changes)
            if (newPort == null || newPort != port) {
                // Xor-Paint the old Connector (Hide old Connector)
                paintConnector(Color.black, graph.getBackground(), g);
                // If Port was found then Point to Port Location
                port = newPort;
                if (port != null)
                    current = graph.toScreen(port.getLocation());
                // Else If no Port was found then Point to Mouse Location
                else
                    current = graph.snap(e.getPoint());
                // Xor-Paint the new Connector
                paintConnector(graph.getBackground(), Color.black, g);
            }
        }

            // Create PopupMenu for the Cell
            //JPopupMenu menu = createPopupMenu(e.getPoint(), cell);
            // Display PopupMenu
            //menu.show(graph, e.getX(), e.getY());
            // Else if in ConnectMode and Remembered Port is Valid
        } else if (port != null && graph.isPortsVisible()) {
            // Remember Start Location
            start = graph.toScreen(port.getLocation());
            // Remember First Port
            firstPort = port;
        } else {
            // Call Superclass
            super.mousePressed(e);
        }
    }

    // Find Port under Mouse and Repaint Connector
    public void mouseDragged(MouseEvent e) {
        // If remembered Start Point is Valid
        if (start != null) {
            // Fetch Graphics from Graph
            Graphics g = graph.getGraphics();
            // Reset Remembered Port
            PortView newPort = getTargetPortAt(e.getPoint());
            // Do not flicker (repaint only on real changes)
            if (newPort == null || newPort != port) {
                // Xor-Paint the old Connector (Hide old Connector)
                paintConnector(Color.black, graph.getBackground(), g);
                // If Port was found then Point to Port Location
                port = newPort;
                if (port != null)
                    current = graph.toScreen(port.getLocation());
                // Else If no Port was found then Point to Mouse Location
                else
                    current = graph.snap(e.getPoint());
                // Xor-Paint the new Connector
                paintConnector(graph.getBackground(), Color.black, g);
            }
        }
        // Call Superclass
        super.mouseDragged(e);
    }

    public PortView getSourcePortAt(Point2D point) {
        // Disable jumping
        graph.setJumpToDefaultPort(false);
        PortView result;
        try {
            // Find a Port View in Model Coordinates and Remember
            result = graph.getPortViewAt(point.getX(), point.getY());
        } finally {
            graph.setJumpToDefaultPort(true);
        }
        return result;
    }

    // Find a Cell at point and Return its first Port as a PortView
    protected PortView getTargetPortAt(Point2D point) {
        // Find a Port View in Model Coordinates and Remember
        return graph.getPortViewAt(point.getX(), point.getY());
    }

    // Connect the First Port and the Current Port in the Graph or Repaint
    public void mouseReleased(MouseEvent e) {
        // If Valid Event, Current and First Port
        if (e != null && port != null && firstPort != null
                && firstPort != port) {
            // Then Establish Connection
            connect((Port) firstPort.getCell(), (Port) port.getCell());
            e.consume();
            // Else Repaint the Graph
        } else
//            graph.repaint();
        // Reset Global Vars
        firstPort = port = null;
        start = current = null;
        // Call Superclass
        super.mouseReleased(e);
    }

    // Show Special Cursor if Over Port
    public void mouseMoved(MouseEvent e) {
        // Check Mode and Find Port
        if (e != null && getSourcePortAt(e.getPoint()) != null
                && graph.isPortsVisible()) {
            // Set Cusor on Graph (Automatically Reset)
            graph.setCursor(new Cursor(Cursor.HAND_CURSOR));
            // Consume Event
            // Note: This is to signal the BasicGraphUI's
            // MouseHandle to stop further event processing.
            e.consume();
        } else
            // Call Superclass
            super.mouseMoved(e);
    }

    // Use Xor-Mode on Graphics to Paint Connector
    protected void paintConnector(Color fg, Color bg, Graphics g) {
        // Set Foreground
        g.setColor(fg);
        // Set Xor-Mode Color
        g.setXORMode(bg);
        // Highlight the Current Port
        paintPort(graph.getGraphics());
        // If Valid First Port, Start and Current Point
        if (firstPort != null && start != null && current != null)
            // Then Draw A Line From Start to Current Point
            g.drawLine((int) start.getX(), (int) start.getY(),
                    (int) current.getX(), (int) current.getY());
    }

    // Use the Preview Flag to Draw a Highlighted Port
    protected void paintPort(Graphics g) {
        // If Current Port is Valid
        if (port != null) {
            // If Not Floating Port...
            boolean o = (GraphConstants.getOffset(port.getAllAttributes()) != null);
            // ...Then use Parent's Bounds
            Rectangle2D r = (o) ? port.getBounds() : port.getParentView()
                    .getBounds();
            // Scale from Model to Screen
            r = graph.toScreen((Rectangle2D) r.clone());
            // Add Space For the Highlight Border
            r.setFrame(r.getX() - 3, r.getY() - 3, r.getWidth() + 6, r
                    .getHeight() + 6);
            // Paint Port in Preview (=Highlight) Mode
            graph.getUI().paintCell(g, port, r, true);
        }
    }

    // Insert a new Edge between source and target
    public void connect(Port source, Port target) {
        // Construct Edge with no label
        DefaultEdge edge = createDefaultEdge();
        if (graph.getModel().acceptsSource(edge, source)
                && graph.getModel().acceptsTarget(edge, target)) {
            // Create a Map thath holds the attributes for the edge
            edge.getAttributes().applyMap(createEdgeAttributes());
            // Insert the Edge and its Attributes
            GraphConstants.setRouting(edge.getAttributes(), GraphConstants.ROUTING_SIMPLE);
            GraphConstants.setLineStyle(edge.getAttributes(), GraphConstants.STYLE_ORTHOGONAL);
            graph.getGraphLayoutCache().insertEdge(edge, source, target);
        }
    }

    // Hook for subclassers
    protected DefaultEdge createDefaultEdge() {
        return new DefaultEdge();
    }

    // Hook for subclassers
    public Map createEdgeAttributes() {
        Map map = new Hashtable();
        // Add a Line End Attribute
        GraphConstants.setLineEnd(map, GraphConstants.ARROW_NONE);
        // Add a label along edge attribute
        GraphConstants.setLabelAlongEdge(map, true);
        return map;
    }

} // End of Editor.MyMarqueeHandler


